﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Windows.ApplicationModel;
using Windows.Storage;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

namespace JukugoQuiz {
  public enum SearchResult { NotFound, Pre, Post }

  public sealed partial class MainPage : Page {
    private TextBlock[] TextBlocks = new TextBlock[16];
    private TextBox TextBoxAnswer;
    private List<Word> Dictionary = new List<Word>();
    private Random Rnd = new Random();
    private Word[] Answers = new Word[8];
    private char AnswerChar;
    private Button[] Buttons = new Button[6];

    public MainPage() {
      this.InitializeComponent();
    }

    protected override async void OnNavigatedTo(
      NavigationEventArgs e) {

      const int cellSize = 130;
      const int myFontSize = 90;
      var myFontFamily = new FontFamily("ＭＳ 明朝");

      for (int i = 0; i < 16; i++) {
        var cd = new ColumnDefinition();
        cd.Width = new GridLength(cellSize);
        GridMain.ColumnDefinitions.Add(cd);

        var rd = new RowDefinition();
        rd.Height = new GridLength(cellSize);
        GridMain.RowDefinitions.Add(rd);
      }

      for (int i = 0; i < 16; i++) {
        var tb = new TextBlock();
        tb.HorizontalAlignment = 
          HorizontalAlignment.Stretch;
        tb.VerticalAlignment = 
          VerticalAlignment.Stretch;
        tb.FontFamily = myFontFamily;
        tb.FontSize = myFontSize;
        tb.Foreground = new SolidColorBrush(
          Windows.UI.Colors.Black);
        tb.Width = cellSize;
        tb.Height = cellSize;

        switch (i) {
          case 8: case 15:
            tb.Text = "↘";
            break;
          case 9: case 14:
            tb.Text = "↓";
            break;
          case 10: case 13:
            tb.Text = "↙";
            break;
          case 11: case 12:
            tb.Text = "→";
            break;
        }

        switch (i) {
          case 0: case 1: case 2:
            tb.SetValue(Grid.ColumnProperty, i * 2);
            tb.SetValue(Grid.RowProperty, 0);
            break;
          case 3:
            tb.SetValue(Grid.ColumnProperty, 0);
            tb.SetValue(Grid.RowProperty, 2);
            break;
          case 4:
            tb.SetValue(Grid.ColumnProperty, 4);
            tb.SetValue(Grid.RowProperty, 2);
            break;
          case 5: case 6: case 7:
            tb.SetValue(Grid.ColumnProperty, 
              (i - 5) * 2);
            tb.SetValue(Grid.RowProperty, 4);
            break;
          case 8: case 9: case 10:
            tb.SetValue(Grid.ColumnProperty, i - 7);
            tb.SetValue(Grid.RowProperty, 1);
            break;
          case 11:
            tb.SetValue(Grid.ColumnProperty, 1);
            tb.SetValue(Grid.RowProperty, 2);
            break;
          case 12:
            tb.SetValue(Grid.ColumnProperty, 3);
            tb.SetValue(Grid.RowProperty, 2);
            break;
          case 13: case 14: case 15:
            tb.SetValue(Grid.ColumnProperty, i - 12);
            tb.SetValue(Grid.RowProperty, 3);
            break;
        }
        GridMain.Children.Add(tb);
        TextBlocks[i] = tb;
      }

      var tba = new TextBox();
      tba.Width = cellSize - 20;
      tba.Height = cellSize - 30;
      tba.HorizontalAlignment = 
        HorizontalAlignment.Left;
      tba.VerticalAlignment = 
        VerticalAlignment.Top;
      tba.FontFamily = myFontFamily;
      tba.FontSize = myFontSize;
      tba.SetValue(Grid.ColumnProperty, 2);
      tba.SetValue(Grid.RowProperty, 2);
      GridMain.Children.Add(tba);
      TextBoxAnswer = tba;

      for (int i = 0; i < 6; i++) {
        var b = new Button();
        switch (i) {
          case 0: case 3:
            b.Content = "これでどうだ！";
            b.Click += ButtonSubmit_Click;
            break;
          case 1: case 4:
            b.Content = "降参です";
            b.Click += ButtonGiveUp_Click;
            break;
          case 2: case 5:
            b.Content = "次の問題";
            b.Click += ButtonNext_Click;
            break;
        }

        if (i < 3) {
          b.SetValue(Grid.RowProperty, i);
          GridRight.Children.Add(b);
        } else {
          b.SetValue(Grid.ColumnProperty, i - 3);
          GridBottom.Children.Add(b);
        }
        Buttons[i] = b;
      }
      await LoadData();
      ShowQuestion();
    }

    private async Task LoadData() {
      var sFolder = await Package.Current.
        InstalledLocation.GetFolderAsync("Data");
      var sFile = await sFolder.GetFileAsync(
        "Jukugo2c_utf8.txt");
      string dicString = 
        await FileIO.ReadTextAsync(sFile);
      var lines = dicString.Split(
        new string[] { Environment.NewLine },
        StringSplitOptions.RemoveEmptyEntries);
      foreach (string line in lines) {
        var columns = line.Split(',');
        var w = new Word(columns[0], columns[1]);
        Dictionary.Add(w);
      }
    }

    private void ShowQuestion() {
      TextBoxAnswer.Text = "";
      int candidateCount = 0;
      string selectedWord = "";
      char selectedChar = ' ';
      var preCandidate = new List<Word>();
      var postCandidate = new List<Word>();

      while (candidateCount < 4) {
        selectedWord = Dictionary[
          Rnd.Next(Dictionary.Count)].Kanji;
        selectedChar = selectedWord[Rnd.Next(2)];
        preCandidate.Clear();
        postCandidate.Clear();

        foreach (Word w in Dictionary)
          switch (w.Search(selectedChar)) {
            case SearchResult.Pre:
              preCandidate.Add(w);
              break;
            case SearchResult.Post:
              postCandidate.Add(w);
              break;
          }
        candidateCount = preCandidate.Count +
          postCandidate.Count;
      }

      for (int i = 0; i < 8; i++)
        Answers[i] = null;

      if (preCandidate.Count > 4)
        for (int i = 0; i < 4; i++) {
          int index = Rnd.Next(preCandidate.Count);
          Answers[i] = preCandidate[index];
          preCandidate.RemoveAt(index);
        } else
        for (int i = 0; i < preCandidate.Count; i++)
          Answers[i] = preCandidate[i];

      if (postCandidate.Count > 4)
        for (int i = 0; i < 4; i++) {
          int index = Rnd.Next(postCandidate.Count);
          Answers[i + 4] = postCandidate[index];
          postCandidate.RemoveAt(index);
        } else
        for (int i = 0; i < postCandidate.Count; i++)
          Answers[i + 4] = postCandidate[i];

      for (int i = 0; i < 8; i++)
        if (Answers[i] == null)
          TextBlocks[i].Text = "●";
        else {
          int startIndex = 1;
          if (i < 4) startIndex = 0;
          TextBlocks[i].Text = Answers[i].
            Kanji[startIndex].ToString();
        }
      AnswerChar = selectedChar;
    }

    private async void ButtonSubmit_Click(
      object sender, RoutedEventArgs e) {

      string s = "";
      if (TextBoxAnswer.Text == AnswerChar.ToString())
        s = "正解です！" + Environment.NewLine + 
          Environment.NewLine + GetAnswerString();
      else
        s = "違います";
      await ShowMessage(s);
    }

    private string GetAnswerString() {
      string s = "";
      for (int i = 0; i < 8; i++) {
        if (Answers[i] == null) continue;
        s += Answers[i].Kanji + "（" + Answers[i].Yomi + 
          "）" + Environment.NewLine;
      }
      return s;
    }

    private async Task ShowMessage(string s) {
      var md = new Windows.UI.Popups.MessageDialog(s);
      await md.ShowAsync();
    }

    private async void ButtonGiveUp_Click(
      object sender, RoutedEventArgs e) {

      TextBoxAnswer.Text = AnswerChar.ToString();
      await ShowMessage("答えは「" + AnswerChar + "」です" +
        Environment.NewLine + Environment.NewLine + 
        GetAnswerString());
    }

    private void ButtonNext_Click(object sender, 
      RoutedEventArgs e) {

      ShowQuestion();
    }

    private void Page_SizeChanged(object sender, 
      SizeChangedEventArgs e) {

      if (this.ActualHeight > this.ActualWidth) {
        GridRight.Visibility = Visibility.Collapsed;
        GridBottom.Visibility = Visibility.Visible;
      } else {
        GridRight.Visibility = Visibility.Visible;
        GridBottom.Visibility = Visibility.Collapsed;
      }
    }
  }
}