﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace MakeDic {
  public partial class Form1 : Form {
    private string WorkFolder = @"C:\work";
    private string[] InputFiles = new string[]{
      "Noun.adjv.dic", "Noun.adverbal.dic",
      "Noun.demonst.dic", "Noun.dic",
      "Noun.nai.dic", "Noun.others.dic",
      "Noun.verbal.dic"};
    private Encoding InEncoding = Encoding.GetEncoding("Shift_JIS");
    private Encoding OutEncoding = Encoding.GetEncoding("UTF-8");

    public Form1() {
      InitializeComponent();
    }

    private void buttonExtract_Click(object sender, EventArgs e) {
      var sb = new StringBuilder();
      foreach(string inputFile in InputFiles)
        sb.Append(ProcessOneFile(Path.Combine(WorkFolder, inputFile)));

      var list1 = sb.ToString().Split(new string[]{Environment.NewLine}, 
        StringSplitOptions.RemoveEmptyEntries);
      var list2 = new List<string>();

      foreach(string line in list1) {
        var columns = line.Split(',');
        if(columns[0].Length == 2) {
          if(Regex.IsMatch(columns[0], @"\p{IsHiragana}")) continue;
          if(Regex.IsMatch(columns[0], @"\p{IsKatakana}")) continue;
          if(Regex.IsMatch(columns[0], 
            "[１２３４５６７８９０ＡＢＣＤＥＦＧＨＩ" +
            "ＪＫＬＭＮＯＰＱＲＳＴＵＶＸＹＺ" +
            "αβγｉｎ〆〇]")) continue;
          if (columns[0][0] == columns[0][1]) continue;
          list2.Add(Regex.Replace(line, "[{}]", ""));
        }
      }

      var shouldRemove = new List<int>();
      list2.Sort();

      for(int i = 0; i < list2.Count; i++)
        for (int j = i + 1; j < list2.Count; j++) {
          if(list2[i] != list2[j] ) break;
          else shouldRemove.Add(j);
        }

      shouldRemove.Sort();
      for (int i = shouldRemove.Count - 1; i > -1; i--)
        list2.RemoveAt(shouldRemove[i]);

      sb.Clear();
      foreach(string line in list2) sb.AppendLine(line);
      File.WriteAllText(Path.Combine(WorkFolder, "Jukugo2c_utf8.txt"), 
        sb.ToString(), OutEncoding);
      MessageBox.Show("処理を終えました");
    }

    private string ProcessOneFile(string fileName) {
      var input = File.ReadAllText(fileName, InEncoding);
      var lines = input.Split(new string[]{Environment.NewLine}, 
        StringSplitOptions.RemoveEmptyEntries);
      var result = new StringBuilder();
      foreach(string line in lines)
        result.AppendLine(Extract(line));
      return result.ToString();
    }

    private string Extract(string s) {
      s = Regex.Replace(s, @"^.*見出し語\s\(", "");
      s = Regex.Replace(s, @"\s.*読み\s", ",");
      s = Regex.Replace(s, @"\).*$", "");
      return s;
    }
  }
}
