﻿Imports Windows.Storage
Imports Windows.UI.Xaml

Public Enum SearchResult
  NotFound
  Pre
  Post
End Enum

Public NotInheritable Class MainPage
  Inherits Page

  Private TextBlocks(15) As TextBlock
  Private TextBoxAnswer As TextBox
  Private Dictionary As New List(Of Word)
  Private Rnd As New Random()
  Private Answers(7) As Word
  Private AnswerChar As Char
  Private Buttons(5) As Button

  Protected Overrides Async Sub OnNavigatedTo(
    e As Navigation.NavigationEventArgs)

    Const cellSize = 130
    Const myFontSize = 90
    Dim myFontFamily = New FontFamily("ＭＳ 明朝")

    For i = 0 To 4
      Dim cd As New ColumnDefinition()
      cd.Width = New GridLength(cellSize)
      GridMain.ColumnDefinitions.Add(cd)

      Dim rd As New RowDefinition()
      rd.Height = New GridLength(cellSize)
      GridMain.RowDefinitions.Add(rd)
    Next

    For i = 0 To 15
      TextBlocks(i) = New TextBlock()
      With TextBlocks(i)
        .FontFamily = myFontFamily
        .FontSize = myFontSize
        .Foreground = New SolidColorBrush(
          Windows.UI.Colors.Black)

        Select Case i
          Case 8, 15
            .Text = "↘"
          Case 9, 14
            .Text = "↓"
          Case 10, 13
            .Text = "↙"
          Case 11, 12
            .Text = "→"
        End Select

        Select Case i
          Case 0 To 2
            .SetValue(Grid.ColumnProperty, i * 2)
            .SetValue(Grid.RowProperty, 0)
          Case 3
            .SetValue(Grid.ColumnProperty, 0)
            .SetValue(Grid.RowProperty, 2)
          Case 4
            .SetValue(Grid.ColumnProperty, 4)
            .SetValue(Grid.RowProperty, 2)
          Case 5 To 7
            .SetValue(Grid.ColumnProperty, (i - 5) * 2)
            .SetValue(Grid.RowProperty, 4)
          Case 8 To 10
            .SetValue(Grid.ColumnProperty, i - 7)
            .SetValue(Grid.RowProperty, 1)
          Case 11
            .SetValue(Grid.ColumnProperty, 1)
            .SetValue(Grid.RowProperty, 2)
          Case 12
            .SetValue(Grid.ColumnProperty, 3)
            .SetValue(Grid.RowProperty, 2)
          Case 13 To 15
            .SetValue(Grid.ColumnProperty, i - 12)
            .SetValue(Grid.RowProperty, 3)
        End Select
      End With
      GridMain.Children.Add(TextBlocks(i))
    Next

    TextBoxAnswer = New TextBox()
    With TextBoxAnswer
      .Width = cellSize - 20
      .Height = cellSize - 30
      .HorizontalAlignment = HorizontalAlignment.Left
      .VerticalAlignment = VerticalAlignment.Top
      .FontFamily = myFontFamily
      .FontSize = myFontSize
      .SetValue(Grid.ColumnProperty, 2)
      .SetValue(Grid.RowProperty, 2)
    End With
    GridMain.Children.Add(TextBoxAnswer)

    For i = 0 To 5
      Buttons(i) = New Button()
      With Buttons(i)
        Select Case i
          Case 0, 3
            .Content = "これでどうだ！"
            AddHandler .Click,
              AddressOf ButtonSubmit_Click
          Case 1, 4
            .Content = "降参です"
            AddHandler .Click,
              AddressOf ButtonGiveUp_Click
          Case 2, 5
            .Content = "次の問題"
            AddHandler .Click,
              AddressOf ButtonNext_Click
        End Select

        Select Case i
          Case 0 To 2
            .SetValue(Grid.RowProperty, i)
            GridRight.Children.Add(Buttons(i))
          Case 3 To 5
            .SetValue(Grid.ColumnProperty, i - 3)
            GridBottom.Children.Add(Buttons(i))
        End Select
      End With
    Next
    Await LoadData()
    ShowQuestion()
  End Sub

  Private Async Function LoadData() As Task
    Dim sFolder = Await Package.Current.
      InstalledLocation.GetFolderAsync("Data")
    Dim sFile = Await sFolder.GetFileAsync(
      "Jukugo2c_utf8.txt")
    Dim dicString = Await FileIO.ReadTextAsync(sFile)
    Dim lines = dicString.Split(
      New String() {vbCrLf},
      StringSplitOptions.RemoveEmptyEntries)
    For Each line In lines
      Dim columns = line.Split(","c)
      Dim w = New Word(columns(0), columns(1))
      Dictionary.Add(w)
    Next
  End Function

  Private Sub ShowQuestion()
    TextBoxAnswer.Text = ""
    Dim candidateCount = 0
    Dim selectedWord As String = ""
    Dim selectedChar As Char
    Dim preCandidate = New List(Of Word)
    Dim postCandidate = New List(Of Word)

    While candidateCount < 4
      selectedWord = Dictionary(
        Rnd.Next(Dictionary.Count)).Kanji
      selectedChar = selectedWord.Substring(
        Rnd.Next(2))
      preCandidate.Clear()
      postCandidate.Clear()
      For Each w In Dictionary
        Select Case w.Search(selectedChar)
          Case SearchResult.Pre
            preCandidate.Add(w)
          Case SearchResult.Post
            postCandidate.Add(w)
        End Select
      Next
      candidateCount = preCandidate.Count +
        postCandidate.Count
    End While

    For i = 0 To 7
      Answers(i) = Nothing
    Next

    If preCandidate.Count > 4 Then
      For i = 0 To 3
        Dim index = Rnd.Next(preCandidate.Count)
        Answers(i) = preCandidate(index)
        preCandidate.RemoveAt(index)
      Next
    Else
      For i = 0 To preCandidate.Count - 1
        Answers(i) = preCandidate(i)
      Next
    End If

    If postCandidate.Count > 4 Then
      For i = 0 To 3
        Dim index = Rnd.Next(postCandidate.Count)
        Answers(i + 4) = postCandidate(index)
        postCandidate.RemoveAt(index)
      Next
    Else
      For i = 0 To postCandidate.Count - 1
        Answers(i + 4) = postCandidate(i)
      Next
    End If

    For i = 0 To 7
      If Answers(i) Is Nothing Then
        TextBlocks(i).Text = "●"
      Else
        Dim startIndex = 1
        If i < 4 Then startIndex = 0
        TextBlocks(i).Text =
          Answers(i).Kanji(startIndex)
      End If
    Next
    AnswerChar = selectedChar
  End Sub

  Private Async Sub ButtonSubmit_Click(
    sender As Object, e As RoutedEventArgs)

    Dim s = ""
    If TextBoxAnswer.Text = AnswerChar.ToString() Then
      s = "正解です！" & vbCrLf & vbCrLf &
        GetAnswerString()
    Else
      s = "違います"
    End If
    Await ShowMessage(s)
  End Sub

  Private Function GetAnswerString() As String
    Dim s = ""
    For i = 0 To 7
      If Answers(i) Is Nothing Then Continue For
      s &= Answers(i).Kanji & "（" &
        Answers(i).Yomi & "）" & vbCrLf
    Next
    Return s
  End Function

  Private Async Function ShowMessage(
    s As String) As task

    Dim md = New Windows.UI.Popups.MessageDialog(s)
    Await md.ShowAsync()
  End Function

  Private Async Sub ButtonGiveUp_Click(
    sender As Object, e As RoutedEventArgs)

    TextBoxAnswer.Text = AnswerChar.ToString()
    Await ShowMessage("答えは「" & AnswerChar & "」です" &
      vbCrLf & vbCrLf & GetAnswerString())
  End Sub

  Private Sub ButtonNext_Click(sender As Object,
    e As RoutedEventArgs)

    ShowQuestion()
  End Sub

  Private Sub Page_SizeChanged(sender As Object,
  e As SizeChangedEventArgs)

    If Me.ActualHeight > Me.ActualWidth Then
      GridRight.Visibility = Visibility.Collapsed
      GridBottom.Visibility = Visibility.Visible
    Else
      GridRight.Visibility = Visibility.Visible
      GridBottom.Visibility = Visibility.Collapsed
    End If
  End Sub
End Class
